import { ChangeDetectionStrategy, Component, EventEmitter, Input, KeyValueDiffers, NgModule, NgZone, Output, ViewChild, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { fromTextArea } from 'codemirror';
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} str
 * @return {?}
 */
function normalizeLineEndings(str) {
    if (!str) {
        return str;
    }
    return str.replace(/\r\n|\r/g, '\n');
}
var CodemirrorComponent = /** @class */ (function () {
    /**
     * @param {?} _differs
     * @param {?} _ngZone
     */
    function CodemirrorComponent(_differs, _ngZone) {
        this._differs = _differs;
        this._ngZone = _ngZone;
        /* name applied to the created textarea */
        this.name = 'codemirror';
        /* autofocus setting applied to the created textarea */
        this.autoFocus = false;
        /* called when the text cursor is moved */
        this.cursorActivity = new EventEmitter();
        /* called when the editor is focused or loses focus */
        this.focusChange = new EventEmitter();
        /* called when the editor is scrolled */
        this.scroll = new EventEmitter();
        this.value = '';
        this.disabled = false;
        this.isFocused = false;
        /**
         * Implemented as part of ControlValueAccessor.
         */
        this.onChange = function (_) { };
        /**
         * Implemented as part of ControlValueAccessor.
         */
        this.onTouched = function () { };
    }
    Object.defineProperty(CodemirrorComponent.prototype, "options", {
        /**
         * set options for codemirror
         * @link http://codemirror.net/doc/manual.html#config
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._options = value;
            if (!this._differ && value) {
                this._differ = this._differs.find(value).create();
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    CodemirrorComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.codeMirror = fromTextArea(this.ref.nativeElement, this._options);
        this._ngZone.runOutsideAngular(function () {
            _this.codeMirror.on('change', _this.codemirrorValueChanged.bind(_this));
            _this.codeMirror.on('cursorActivity', _this.cursorActive.bind(_this));
            _this.codeMirror.on('focus', _this.focusChanged.bind(_this, true));
            _this.codeMirror.on('blur', _this.focusChanged.bind(_this, false));
            _this.codeMirror.on('scroll', _this.scrollChanged.bind(_this));
        });
    };
    /**
     * @return {?}
     */
    CodemirrorComponent.prototype.ngDoCheck = function () {
        var _this = this;
        if (this._differ) {
            // check options have not changed
            var /** @type {?} */ changes = this._differ.diff(this._options);
            if (changes) {
                changes.forEachChangedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
                changes.forEachAddedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
                changes.forEachRemovedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
            }
        }
    };
    /**
     * @return {?}
     */
    CodemirrorComponent.prototype.ngOnDestroy = function () {
        // is there a lighter-weight way to remove the cm instance?
        if (this.codeMirror) {
            this.codeMirror.toTextArea();
        }
    };
    /**
     * @param {?} doc
     * @param {?} change
     * @return {?}
     */
    CodemirrorComponent.prototype.codemirrorValueChanged = function (doc, change) {
        if (change.origin !== 'setValue') {
            this.value = doc.getValue();
            this.writeValue(doc.getValue());
        }
    };
    /**
     * @param {?} optionName
     * @param {?} newValue
     * @return {?}
     */
    CodemirrorComponent.prototype.setOptionIfChanged = function (optionName, newValue) {
        if (!this.codeMirror) {
            return;
        }
        this.codeMirror.setOption(optionName, newValue);
    };
    /**
     * @param {?} focused
     * @return {?}
     */
    CodemirrorComponent.prototype.focusChanged = function (focused) {
        this.onTouched();
        this.isFocused = focused;
        this.focusChange.emit(focused);
    };
    /**
     * @param {?} cm
     * @return {?}
     */
    CodemirrorComponent.prototype.scrollChanged = function (cm) {
        this.scroll.emit(cm.getScrollInfo());
    };
    /**
     * @param {?} cm
     * @return {?}
     */
    CodemirrorComponent.prototype.cursorActive = function (cm) {
        this.cursorActivity.emit(cm);
    };
    /**
     * Implemented as part of ControlValueAccessor.
     * @param {?} value
     * @return {?}
     */
    CodemirrorComponent.prototype.writeValue = function (value) {
        if (value === null) {
            return;
        }
        if (value &&
            value !== this.codeMirror.getValue() &&
            normalizeLineEndings(this.codeMirror.getValue()) !==
                normalizeLineEndings(value)) {
            this.value = value;
            if (this.preserveScrollPosition) {
                var /** @type {?} */ prevScrollPosition = this.codeMirror.getScrollInfo();
                this.codeMirror.setValue(this.value);
                this.codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
                return;
            }
            this.codeMirror.setValue(this.value);
            // Don't call onChange value is from ngModel
            return;
        }
        this.onChange(this.value);
    };
    /**
     * Implemented as part of ControlValueAccessor.
     * @param {?} fn
     * @return {?}
     */
    CodemirrorComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    /**
     * Implemented as part of ControlValueAccessor.
     * @param {?} fn
     * @return {?}
     */
    CodemirrorComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    /**
     * Implemented as part of ControlValueAccessor.
     * @param {?} isDisabled
     * @return {?}
     */
    CodemirrorComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.setOptionIfChanged('readOnly', this.disabled);
    };
    return CodemirrorComponent;
}());
CodemirrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-codemirror',
                template: "\n  <textarea\n    [name]=\"name\"\n    class=\"ngx-codemirror {{ className }}\"\n    [class.ngx-codemirror--focused]=\"isFocused\"\n    autocomplete=\"off\"\n    [autofocus]=\"autoFocus\"\n    #ref>\n  </textarea>\n  ",
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(function () { return CodemirrorComponent; }),
                        multi: true,
                    },
                ],
                changeDetection: ChangeDetectionStrategy.OnPush,
            },] },
];
/** @nocollapse */
CodemirrorComponent.ctorParameters = function () { return [
    { type: KeyValueDiffers, },
    { type: NgZone, },
]; };
CodemirrorComponent.propDecorators = {
    "className": [{ type: Input },],
    "name": [{ type: Input },],
    "autoFocus": [{ type: Input },],
    "options": [{ type: Input },],
    "preserveScrollPosition": [{ type: Input },],
    "cursorActivity": [{ type: Output },],
    "focusChange": [{ type: Output },],
    "scroll": [{ type: Output },],
    "ref": [{ type: ViewChild, args: ['ref',] },],
};
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var CodemirrorModule = /** @class */ (function () {
    function CodemirrorModule() {
    }
    return CodemirrorModule;
}());
CodemirrorModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [CodemirrorComponent],
                declarations: [CodemirrorComponent],
                providers: [],
            },] },
];
/** @nocollapse */
CodemirrorModule.ctorParameters = function () { return []; };
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */
export { CodemirrorComponent, CodemirrorModule };
//# sourceMappingURL=ctrl-ngx-codemirror.js.map
